"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var URI_1 = require("../util/URI");
var SassImplementation_1 = require("../util/SassImplementation");
var IS_WINDOWS = /win32/.test(require("os").platform());
var normalizeURI = function (_eyeglass, sass) {
    var methods = {
        "eyeglass-uri-preserve($uri)": function ($uri, done) {
            if (!SassImplementation_1.isSassString(sass, $uri)) {
                return done(SassImplementation_1.typeError(sass, "string", $uri));
            }
            var uri = $uri.getValue();
            // decorate the uri
            uri = URI_1.URI.preserve(uri);
            done(sass.types.String(uri));
        },
        "eyeglass-uri-restore($uri)": function ($uri, done) {
            if (!SassImplementation_1.isSassString(sass, $uri)) {
                return done(SassImplementation_1.typeError(sass, "string", $uri));
            }
            var uri = $uri.getValue();
            // restore the uri
            uri = URI_1.URI.restore(uri);
            done(sass.types.String(uri));
        }
    };
    if (IS_WINDOWS || process.env.EYEGLASS_NORMALIZE_PATHS) {
        methods["-eyeglass-normalize-uri($uri, $type: web)"] = function ($uri, $type, done) {
            if (!SassImplementation_1.isSassString(sass, $type)) {
                return done(SassImplementation_1.typeError(sass, "string", $type));
            }
            if (!SassImplementation_1.isSassString(sass, $uri)) {
                return done(SassImplementation_1.typeError(sass, "string", $uri));
            }
            var type = $type.getValue();
            var uri = $uri.getValue();
            // normalize the uri for the given type
            uri = URI_1.URI[type](uri);
            done(sass.types.String(uri));
        };
    }
    return methods;
};
exports.default = normalizeURI;
//# sourceMappingURL=normalize-uri.js.map